# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# util.py
#
# fonctions utilitaires pour la librairie sso eole
# (actuellement seulement annuaire LDAP)
#
###########################################################################
import urlparse, socket, re, IPy
import time, random, sha
from twisted.web2.http_headers import parseCookie
from twisted.python import log

def urljoin(url, params):
    """ajoute des paramètres à une url
    >>> urljoin('http://test/','a=b')
    'http://test/?a=b'
    >>> urljoin('http://test/?a=b','c=d')
    'http://test/?a=b&c=d'
    """
    urllist = list(urlparse.urlsplit(url))
    if urllist[-2] == '':
        urllist[-2] = params
    else:
        urllist[-2] += '&' + params
    return urlparse.urlunsplit(urllist)

head = """<head>
<meta http-equiv="Pragma" content="no-cache">
<meta http-equiv="Cache-Control" content="no-store">
<meta http-equiv='Expires' content='Mon, 26 Jul 1997 05:00:00 GMT'>
<title>Authentification</title>
<link rel='stylesheet' type='text/css' href='/css/leaves.css'>
</head>
"""

def cas_response(resptype, body, **attrs):
    """convenience cas response factory"""
    attrs = ' '.join('%s="%s"' % kv for kv in attrs.items())
    if attrs:
        attrs = ' ' + attrs
    return str("""<?xml version="1.0" encoding="UTF-8"?>
<cas:serviceResponse xmlns:cas="http://www.yale.edu/tp/cas">
  <cas:%(rtype)s%(attrs)s>
    %(body)s
  </cas:%(rtype)s>
</cas:serviceResponse>\r\n\r\n""" % \
{'rtype': resptype, 'attrs': attrs, 'body': body})

def unescape(url, quote=None):
    """supprime les caractères escapés dans une url"""
    entities = [('&gt;','>'), ('&lt;','<'), ('&amp;','&')]
    if quote:
        entities.insert(0, ('&quot;','"'))
    for substr, car in entities:
        url = url.replace(substr, car)
    return url

def get_service_from_url(url):
    """supprime la partie requête d'une url pour permettre d'indentifier le service
    """
    url = unescape(url)
    u = urlparse.urlparse(url)
    service = "%s://%s%s" % (u.scheme, u.netloc, u.path)
    return service

def getCookies(ctx):
    """renvoie la liste  des cookies sous forme Twisted"""
    cookiestr = ctx.headers.getRawHeaders('Cookie')
    if cookiestr:
        return parseCookie(cookiestr)
    return []

def getCookie(ctx, key):
    """renvoie un cookie Twisted"""
    for cookie in getCookies(ctx):
        if cookie.name == key:
            return cookie
    return None

def verif_cert(conn, cert, err_num, err_depth, rc):
    """fonction de vérification du certificat"""
    return not cert.has_expired()

def check_hostname_by_ip(url, addr):
    """vérification de la validité de l'hôte de l'url en fonction d'une IP
    """
    try:
        # vérification de l'adresse
        ip = IPy.IP(addr)
        try:
            hostname = socket.gethostbyname(url.hostname)
        except:
            hostname = url.hostname
        assert ip.overlaps(hostname)
        return True
    except:
        return False
    return False

def check_hostname_by_domain(url, addr):
    """vérification de la validité de l'hôte de l'url par expression régulière
    """
    #try:
    #    hostname = socket.gethostbyaddr(url.hostname)[0]
    #except:
    hostname = url.hostname
    # on essaye un match de l'expression régulière sur le nom d'hôte
    try:
        match_obj = re.match(addr, hostname)
    except:
        log.msg("""Erreur lors du test sur l'expression régulière '%s' (syntaxe invalide ?)""" % addr)
        return False

    return match_obj is not None

def gen_ticket_id(prefix, address):
    """à redéfinir dans la classe dérivée"""
    random_id = str(time.time()).split('.')[0]
    ticket_id = prefix + '-' + address + '-' + sha.sha('%s/%s' % (random_id, random.randrange(2**100))).hexdigest()
    return ticket_id

